package hr.fer.diplomski.seminar.pso;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class Particle {

	private static final double chi = 0.792;
	
	/** Dimension of the vector space */
	private int dim;
	
	/** Particle's current position	 */
	private double[] x;
	
	/** Particle's current speed */
	private double[] v;
	
	/** Particle's best known position */
	private double[] p;
	/** Particle's previous position
	 *  To be used in next versions for 
	 *  graphics visualization.   
	*/
	private double[] xPrev;

	/** Current fitness */
	private double fit;
	/** Best known fitness */
	private double bestFit;
	
	/** Fitness function */
	private Function fitness;
	
	private double vMax;
	private double xMax;
	private double xMin;

	private double c1;
	private double c2;
	private Random rand;
	/** Particle's list of neighbours  */
	private List<Particle> neighbours;
	
	
	/**
	 * Particle's constructor
	 * 
	 * @param eval fitness function
	 * @param xMax maximum range in the space
	 * @param xMin minimum range in the sapce
	 * @param vMax maximum speed (absolute value)
	 * @param rand random number generator
	 * @param c1   user defined constant defines weight factor of personal best 
	 * @param c2   user defined constant defines weight of local best
	 */
	public Particle(Function fitness, double xMax, double xMin, double vMax,
			Random rand, double c1, double c2) {
		super();
		this.fitness = fitness;
		this.xMax = xMax;
		this.xMin = xMin;
		this.vMax = vMax;
		this.rand = rand;
		this.c1 = c1;
		this.c2 = c2;
		//this.neighbours = new ArrayList<Particle>();

		this.dim = fitness.getDim();

		x = new double[dim];
		v = new double[dim];
		p = new double[dim];
		xPrev = new double[dim];

		for (int i = 0; i < dim; i++) {
			x[i] = xMin + (xMax - xMin) * this.rand.nextDouble();
			v[i] = 0.0;
			//v[i] = vMin + (vMax - vMin) * rand.nextDouble();
			p[i] = x[i];
			xPrev[i] = x[i];
		}
		
		fit = fitness.evaluate(x);
		bestFit = fit;
	}
	
	public void addNeighbours(List <Particle> list){
		neighbours = new ArrayList<Particle>(list);
	}
	
	/**
	 * Function determines best neighbour
	 * @return index of the best neighbour in the neighbourhood list
	 */
	public int detBestNeighbour(){
		int bestIdx=0;
		//System.out.println("TEST: neighbourhood size = " + neighbours.size());
		double bestNeighbourhoodFit = neighbours.get(0).getBestFit();
		for (int i =0; i < neighbours.size(); i++){
			if (neighbours.get(i).getBestFit() < bestNeighbourhoodFit){
				bestIdx = i;
			}
		}
		return bestIdx;
	}
	
	/**
	 * 
	 * Method does one particle's iteration:
	 *  1. updates it speed
	 *  2. updates it's position
	 *  3. updates best known position
	 */
	public void iteration(){
		Particle bestNeighbour = neighbours.get(detBestNeighbour());
		double [] bestNeigPos = bestNeighbour.getBestPosition();
		for (int i =0; i < dim; i++){
			//update previous position
			xPrev[i] = x[i];
			//update current speed
			v[i]= chi *  (v[i] + rand.nextDouble() * c1 * (p[i] - x[i]) +
					rand.nextDouble() * c2 * (bestNeigPos[i] - x[i]));
			
			// don't let the spped get out of range
			v[i] = v[i] > vMax ? vMax:v[i];
			v[i] = v[i] < -vMax ? -vMax : v[i];
			
			//update current position
			x[i] = x[i] + v[i];
			
			// don't let x[i] get out of the range!
			x[i] = x[i] > xMax ? xMax : x[i];
			x[i] = x[i] < xMin ? xMin : x[i];
		
			
		}
		
		fit = fitness.evaluate(x);
		if (fit < bestFit){
			bestFit = fit;
			for (int i =0; i < dim; i++){
				p[i] = x[i];
			}
		}
		
	}
	public double [] getBestPosition(){
		return p;
	}
	public double getBestFit(){
		return this.bestFit;
	}
}
